/*****************************************************************

Copyright (c) 2004 Abdiel Janulgue <xynopsis@yahoo.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 
******************************************************************/

// Simple Kiosk Lock Protocol client implementation
#ifndef __KIOSKLOCKP_H_
#define __KIOSKLOCKP_H_

#include <qtextstream.h>
#include <qsocket.h>
#include <qobject.h>
#include <qmessagebox.h> 
#include <qstring.h>

class KioskLockP: public QObject
{
    Q_OBJECT

public:
    KioskLockP(const QString & h, Q_UINT16 pt, QWidget* p) 
    {
	host = h;
	port = pt;
	boxparent = p;
	socket = new QSocket( this );
	connect( socket, SIGNAL(readyRead()), SLOT(read()) );
        connect( socket, SIGNAL(error(int)), SLOT(error(int)) );
    } 
    ~KioskLockP() { delete socket; }
    void sendSerial(const QString & serial)
    {
	socket->connectToHost( host, port );
	QTextStream os(socket);
	os << "POST " << serial << endl;
    }

    void sendInvalidate(const QString & serial)
    {
	socket->connectToHost( host, port );
	QTextStream os(socket);
	os << "INV " << serial << endl;
    }
    
    void sendUpdate(const QString & serial, int minutes)
    {
	socket->connectToHost( host, port );
	QTextStream os(socket);
	os << "UPD " << serial << " " << minutes << endl;
    }
signals:
	void getminutes(int);

private slots: 
    void read() 
    {       
	while(socket->canReadLine() ) {
	    int amt = socket->readLine().toInt();
	    // qDebug("read received");
	    // handle confirmation returns
	    if(amt > 1) 
		emit getminutes(amt * 60);
	    // TODO return code on server if some DB modification fails
	    // else 
	    //	emit getminutes(amt);
	}
	socket->close(); 
    }           
    void error(int error) 
    {
	QString err("Error in connecting to openkiosk server, ");
	switch(error) {
	case QSocket::ErrConnectionRefused:
	    err += "the connection was refused. ";
	    break;
	case QSocket::ErrHostNotFound:
	    err += "the host was not found. ";
	    break;
	case QSocket::ErrSocketRead:
	    err += "a read from the socket failed. ";
	    break;
	default:
	    break;
	}
	err += "\nPlease check your network connection.";
	QMessageBox::critical(boxparent , "Error", err); 
    }
private:    
    QString host;
    Q_UINT16 port;
    QSocket* socket;
    QWidget* boxparent;
};

#endif
