/*****************************************************************

Copyright (c) 2004 Abdiel Janulgue <xynopsis@yahoo.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 
******************************************************************/
#include "netaccess.h"

#include <qvariant.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qapplication.h>

#include <X11/Xutil.h>
#include <X11/keysym.h>

/* 
 *  Constructs a NetAccess as a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
NetAccess::NetAccess( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    if ( !name )
	setName( "NetAccess" );
    NetAccessLayout = new QHBoxLayout( this, 11, 6, "NetAccessLayout"); 

    netlabel = new QLabel( this, "netlabel" );
    NetAccessLayout->addWidget( netlabel );

    netedit = new QLineEdit( this, "netedit" );
    netedit->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, netedit->sizePolicy().hasHeightForWidth() ) );
    NetAccessLayout->addWidget( netedit );
    QSpacerItem* spacer = new QSpacerItem( 10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    NetAccessLayout->addItem( spacer );

    buttonlayout = new QVBoxLayout( 0, 0, 6, "buttonlayout"); 
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 41, QSizePolicy::Minimum, QSizePolicy::Expanding );
    buttonlayout->addItem( spacer_2 );

    ok = new QPushButton( this, "ok" );
    buttonlayout->addWidget( ok );

    cancel = new QPushButton( this, "cancel" );
    buttonlayout->addWidget( cancel );
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 41, QSizePolicy::Minimum, QSizePolicy::Expanding );
    buttonlayout->addItem( spacer_3 );
    NetAccessLayout->addLayout( buttonlayout );
    languageChange();
    resize( QSize(308, 102).expandedTo(minimumSizeHint()) );

    // signals and slots connections
    connect( cancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( ok, SIGNAL( clicked() ), this, SLOT( accept() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
NetAccess::~NetAccess()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void NetAccess::languageChange()
{
    setCaption( tr( "Enter your Network Access Number:" ) );
    netlabel->setText( tr( "Access Number:" ) );
    ok->setText( tr( "OK" ) );
    cancel->setText( tr( "Cancel" ) );
}
// stolen from KDE desktop lock - get Keyboard input without calling 
// grabKeyboard()
void NetAccess::show()
{
    QDialog::show();
    raise();
    QApplication::flushX();
    for(;;) { 
	// wait for the window to get mapped
	XWindowAttributes attrs;
	if( XGetWindowAttributes( qt_xdisplay(), winId(), &attrs )
	    && attrs.map_state != IsUnmapped )
	    break;
    }
    setActiveWindow();
    setFocus();
}
