/*****************************************************************

Copyright (c) 2004 Abdiel Janulgue <xynopsis@yahoo.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 
******************************************************************/
#include "x11lock.h"
#include "network"

#include <kwin.h>
#include <qvariant.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qapplication.h> 
#include <qtextstream.h>
#include <qsocket.h>
#include <qobject.h>
#include <qmessagebox.h> 
#include <qstring.h>
#include <qcursor.h>

#include <X11/Xlib.h>
#include <unistd.h>
#include "netaccess.h"
#include "kiosklockp.h"
#include "kiosk.xpm"

x11lock::x11lock( QWidget* parent, const char* name, WFlags fl )
    : QWidget(parent, name, fl ),  isDialogUp(false)
{
    // try to tell window manager we will always be on top
    KWin::setState( winId(), NET::StaysOnTop );
    KWin::setOnAllDesktops( winId(), true );
}   

void x11lock::createWindow()
{
    setPaletteBackgroundColor( QColor( 0, 0, 0 ) );
    x11lockLayout = new QHBoxLayout( this, 11, 6, "x11lockLayout"); 
    
    kioskimage = new QLabel( this, "kioskimage" );
    kioskimage->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, kioskimage->sizePolicy().hasHeightForWidth() ) );
    kioskimage->setPixmap( QPixmap(kiosk) );
    kioskimage->setScaledContents( TRUE );
    x11lockLayout->addWidget( kioskimage );
    
    languageChange();
    getserial = new NetAccess(this, "netaccess");
    
    klp = new KioskLockP( QString(SERV_IP), SERV_PORT, this);
    connect(klp, SIGNAL(getminutes(int)), SLOT(unlock(int)) );

    savetm = new QTimer( this );
    connect( savetm, SIGNAL(timeout()), this, SLOT(savestate()) );
    cdowntm= new QTimer( this );
    connect( cdowntm , SIGNAL(timeout()), this, SLOT(countdown()) );
    connect( &xtimer, SIGNAL( timeout() ), this, SLOT( lock() ) );

    connect(this, SIGNAL(clicked()), this, SLOT(useraction()) );
    connect(this, SIGNAL(released()), this, SLOT(useraction()) );

    XSetWindowAttributes attr;
    attr.event_mask = KeyPressMask | ButtonPressMask | PointerMotionMask |
                        VisibilityChangeMask | ExposureMask;
    XChangeWindowAttributes(qt_xdisplay(), winId(),
                            CWEventMask | CWColormap, &attr);

    QDesktopWidget *d = QApplication::desktop();
    int w = d->width();     // returns desktop width
    int h = d->height();    // returns desktop height
    setGeometry(0, 0, w, h);
    hide();

}

void x11lock::lockscreen()
{
    createWindow(); // create and hide the window --^
    move(0, 0);   // move to top left corner
    show();
    raise();
    setActiveWindow();
    XSync(qt_xdisplay(), False);
}


/*
 *  Destroys the object and frees any allocated resources
 */
x11lock::~x11lock()
{
    delete klp;
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void x11lock::languageChange()
{
    setCaption( tr( "x11lock" ) );
}

// TODO: single shot timer is still running. It is likely that
// it will have disastrous consequences once it wakes up while the screen
// is still locked
void x11lock::update()
{
    if(seconds_copy >= 60)
	klp->sendUpdate(serialnum, seconds_copy);
    else
	klp->sendInvalidate(serialnum);
    lock();
}

#define GRABEVENTS ButtonPressMask | ButtonReleaseMask | PointerMotionMask | \
		   EnterWindowMask | LeaveWindowMask


void x11lock::useraction()
{ 
    XChangeActivePointerGrab( qt_xdisplay(), GRABEVENTS,
			      arrowCursor.handle(), CurrentTime);

    getserial->netedit->clear();
    if(getserial->exec() == QDialog::Accepted) {
	QString val = getserial->netedit->text();
	if(val.length() > 9) {
	    klp->sendSerial(val);
	    serialnum = val;
	}
    }
    
    XChangeActivePointerGrab( qt_xdisplay(), GRABEVENTS,
			      blankCursor.handle(), CurrentTime);
}

void x11lock::screenTimerLock()
{
    raise();
    savetm->stop();
    cdowntm->stop();
    xtimer.stop();
    
    seconds_copy = 0;
    savestate();

    show();

    switch(QMessageBox::information(this , 
	      "Time Out", "Would you like to extend your time?",
	      QMessageBox::Yes,QMessageBox::No )) {
    case QMessageBox::Yes:
	useraction();
	break;
    case QMessageBox::No:
	break;
    }
}

//  TODO:
void x11lock::lock()
{
    klp->sendInvalidate(serialnum);
    screenTimerLock();
}

void x11lock::lockUpdate()
{
    netUpdate(serialnum, seconds_copy);
    screenTimerLock();
}

void x11lock::remainingCheck()
{
    unsigned int minutes = seconds_copy / 60;
    unsigned int hhour = minutes / 60;
    unsigned int mmins = minutes - (hhour * 60);

    QString rem;
   
    if(hhour > 0)
	rem.sprintf("You have %d hour(s) and %d minutes left", hhour, mmins);
    else if( (hhour == 0 ) && (mmins == 60) )
	rem.sprintf("You have 1 minute left. Please Reload soon.\n"
		    "Always save your work to prevent interruptions" );
    else 
	rem.sprintf("You have %d minute(s) left",  mmins);
    QMessageBox::information(this , "Your Remaining Time:", rem); 
}
    

void x11lock::unlock(int seconds)
{
    seconds_copy = seconds;
    // QTimer::singleShot( seconds * 1000, this, SLOT(lock()) );
    xtimer.start( seconds * 1000 );

    // Save info to disk every 5 seconds
    savetm->start(5000);
    // Have a working count down timer 
    cdowntm->start(1000);
    savestate();
    hide();
    lower();
    
    // display info
    remainingCheck();
}

void x11lock::savestate()
{
    QFile file( ".kioskdata" );
    if(file.open( IO_WriteOnly )) {
        QDataStream stream( &file ); 
	stream << serialnum;   // serialize a string
	stream << (Q_INT32)seconds_copy;
	file.close();
    } else 
	QMessageBox::critical(this, "Error","Error saving user data.\
                                  Check file permissions!"); 
}

void x11lock::netUpdate(const QString& s , unsigned int secs)
{   
    klp->sendUpdate(s, secs / 60);
}

void x11lock::countdown()
{
    seconds_copy--;    
    if(seconds_copy == 60)
	remainingCheck();
}

//---------------------------------------------------------------------------
// 
// Grab Input. 
// Idea and code portions from KDE desktop lock :-)
//
// We can't use QWidget's grabKeyboard() and grabMouse() since it prevents us 
// from properly enabling input to the dialog to get input focus 
bool x11lock::grabKeyboard()
{
    int rv = XGrabKeyboard( qt_xdisplay(), QApplication::desktop()->winId(),
        True, GrabModeAsync, GrabModeAsync, CurrentTime );

    return (rv == GrabSuccess);
}

#define GRABEVENTS ButtonPressMask | ButtonReleaseMask | PointerMotionMask | \
		   EnterWindowMask | LeaveWindowMask

bool x11lock::grabMouse()
{
    int rv = XGrabPointer( qt_xdisplay(), QApplication::desktop()->winId(),
            True, GRABEVENTS, GrabModeAsync, GrabModeAsync, None,
            arrowCursor.handle(), CurrentTime );

    return (rv == GrabSuccess);
}


bool x11lock::grabInput()
{
    XSync(qt_xdisplay(), False);
    
    if (!grabKeyboard()) {
        sleep(1);
        if (!grabKeyboard()) {
	    return false;
	}
    }

    if (!grabMouse()) {
        sleep(1);
        if (!grabMouse()) {
            XUngrabKeyboard(qt_xdisplay(), CurrentTime);
            return false;
	}
    }
 
    return true;
}
    
void x11lock::ungrabInput()
{
    XUngrabKeyboard(qt_xdisplay(), CurrentTime);
    XUngrabPointer(qt_xdisplay(), CurrentTime);
}

void x11lock::showEvent ( QShowEvent * )
{
    grabInput();
}
void x11lock::hideEvent ( QHideEvent * )
{
    ungrabInput();
}

//---------------------------------------------------------------------------
//
// X11 Event.
//
bool x11lock::x11Event(XEvent *event)
{
    switch (event->type) 
	{
        case VisibilityNotify:
            if (event->xvisibility.state != VisibilityUnobscured &&
                event->xvisibility.window == winId())
            {
                raise();
		grabInput();
                QApplication::flushX();
            }
            break;

        case ConfigureNotify:
            raise();
	    grabInput();
            QApplication::flushX();
            break;
	}

    return false;
}
